@@ -24,6 +24,31 @@ from automaton import _utils as utils
 from automaton import exceptions as excp
 
 
+class State(object):
+    """Container that defines needed components of a single state.
+
+    Usage of this and the :meth:`~.FiniteMachine.build` make creating finite
+    state machines that much easier.
+
+    :ivar name: The name of the state.
+    :ivar is_terminal: Whether this state is terminal (or not).
+    :ivar next_states: Dictionary of 'event' -> 'next state name' (or none).
+    """
+
+    def __init__(self, name, is_terminal=False, next_states=None):
+        self.name = name
+        self.is_terminal = bool(is_terminal)
+        self.next_states = next_states
+
+
+def _convert_to_states(state_space):
+    # NOTE(harlowja): if provided dicts, convert them...
+    for state in state_space:
+        if isinstance(state, dict):
+            state = State(**state)
+        yield state
+
+
 def _orderedkeys(data, sort=True):
     if sort:
         return sorted(six.iterkeys(data))
@@ -60,17 +85,18 @@ class FiniteMachine(object):
     transition. The other way to use this
     state machine is to skip using  :py:meth:`~automaton.runners.Runner.run`
     or :py:meth:`~automaton.runners.Runner.run_iter`
-    completely and use the :meth:`.process_event` method explicitly and
-    trigger the events via some *external* functionality/triggers...
+    completely and use the :meth:`~.FiniteMachine.process_event` method
+    explicitly and trigger the events via
+    some *external* functionality/triggers...
     """
 
-    # Result of processing an event (cause and effect...)
-    _Effect = collections.namedtuple('_Effect', 'reaction,terminal')
+    #: The result of processing an event (cause and effect...)
+    Effect = collections.namedtuple('Effect', 'reaction,terminal')
 
     @classmethod
     def _effect_builder(cls, new_state, event):
-        return cls._Effect(new_state['reactions'].get(event),
-                           new_state["terminal"])
+        return cls.Effect(new_state['reactions'].get(event),
+                          new_state["terminal"])
 
     @removals.removed_kwarg('default_start_state',
                             message="The usage of 'default_start_state' via"
@@ -104,8 +130,29 @@ class FiniteMachine(object):
                                 " undefined state '%s'" % (state))
         self._default_start_state = state
 
+    @classmethod
+    def build(cls, state_space):
+        """Builds a machine from a state space listing.
+
+        Each element of this list must be an instance
+        of :py:class:`.State` or a ``dict`` with equivalent keys that
+        can be used to construct a :py:class:`.State` instance.
+        """
+        state_space = list(_convert_to_states(state_space))
+        m = cls()
+        for state in state_space:
+            m.add_state(state.name, terminal=state.is_terminal)
+        for state in state_space:
+            if state.next_states:
+                for event, next_state in six.iteritems(state.next_states):
+                    if isinstance(next_state, State):
+                        next_state = next_state.name
+                    m.add_transition(state.name, next_state, event)
+        return m
+
     @property
     def current_state(self):
+        """The current state the machine is in (or none if not initialized)."""
         if self._current is not None:
             return self._current.name
         return None
@@ -120,11 +167,11 @@ class FiniteMachine(object):
     def add_state(self, state, terminal=False, on_enter=None, on_exit=None):
         """Adds a given state to the state machine.
 
-        The on_enter and on_exit callbacks, if provided will be expected to
-        take two positional parameters, these being the state being exited (for
-        on_exit) or the state being entered (for on_enter) and a second
-        parameter which is the event that is being processed that caused the
-        state transition.
+        The ``on_enter`` and ``on_exit`` callbacks, if provided will be
+        expected to take two positional parameters, these being the state
+        being exited (for ``on_exit``) or the state being entered (for
+        ``on_enter``) and a second parameter which is the event that is
+        being processed that caused the state transition.
         """
         if self.frozen:
             raise excp.FrozenMachine()
@@ -186,8 +233,17 @@ class FiniteMachine(object):
             raise excp.Duplicate("State '%s' reaction to event '%s'"
                                  " already defined" % (state, event))
 
-    def add_transition(self, start, end, event):
-        """Adds an allowed transition from start -> end for the given event."""
+    def add_transition(self, start, end, event, replace=False):
+        """Adds an allowed transition from start -> end for the given event.
+
+        :param start: starting state
+        :param end: ending state
+        :param event: event that causes start state to
+                      transition to end state
+        :param replace: replace existing event instead of raising a
+                        :py:class:`~automaton.exceptions.Duplicate` exception
+                        when the transition already exists.
+        """
         if self.frozen:
             raise excp.FrozenMachine()
         if start not in self._states:
@@ -202,9 +258,22 @@ class FiniteMachine(object):
             raise excp.InvalidState("Can not add a transition on event '%s'"
                                     " that starts in the terminal state '%s'"
                                     % (event, start))
-        self._transitions[start][event] = _Jump(end,
-                                                self._states[end]['on_enter'],
-                                                self._states[start]['on_exit'])
+        if event in self._transitions[start] and not replace:
+            target = self._transitions[start][event]
+            if target.name != end:
+                raise excp.Duplicate("Cannot add transition from"
+                                     " '%(start_state)s' to '%(end_state)s'"
+                                     " on event '%(event)s' because a"
+                                     " transition from '%(start_state)s'"
+                                     " to '%(existing_end_state)s' on"
+                                     " event '%(event)s' already exists."
+                                     % {'existing_end_state': target.name,
+                                        'end_state': end, 'event': event,
+                                        'start_state': start})
+        else:
+            target = _Jump(end, self._states[end]['on_enter'],
+                           self._states[start]['on_exit'])
+            self._transitions[start][event] = target
 
     def _pre_process_event(self, event):
         current = self._current
@@ -225,7 +294,17 @@ class FiniteMachine(object):
         return result
 
     def process_event(self, event):
-        """Trigger a state change in response to the provided event."""
+        """Trigger a state change in response to the provided event.
+
+        :returns: Effect this is either a :py:class:`.FiniteMachine.Effect` or
+                  an ``Effect`` from a subclass of :py:class:`.FiniteMachine`.
+                  See the appropriate named tuple for a description of the
+                  actual items in the tuple. For
+                  example, :py:class:`.FiniteMachine.Effect`'s
+                  first item is ``reaction``: one could invoke this reaction's
+                  callback to react to the new stable state.
+        :rtype: namedtuple
+        """
         self._pre_process_event(event)
         current = self._current
         replacement = self._transitions[current.name][event]
@@ -238,7 +317,13 @@ class FiniteMachine(object):
         return self._post_process_event(event, result)
 
     def initialize(self, start_state=None):
-        """Sets up the state machine (sets current state to start state...)."""
+        """Sets up the state machine (sets current state to start state...).
+
+        :param start_state: explicit start state to use to initialize the
+                            state machine to. If ``None`` is provided then
+                            the machine's default start state will be used
+                            instead.
+        """
         if start_state is None:
             start_state = self._default_start_state
         if start_state not in self._states:
@@ -364,9 +449,9 @@ class FiniteMachine(object):
 class HierarchicalFiniteMachine(FiniteMachine):
     """A fsm that understands how to run in a hierarchical mode."""
 
-    # Result of processing an event (cause and effect...)
-    _Effect = collections.namedtuple('_Effect',
-                                     'reaction,terminal,machine')
+    #: The result of processing an event (cause and effect...)
+    Effect = collections.namedtuple('Effect',
+                                    'reaction,terminal,machine')
 
     def __init__(self, default_start_state=None):
         super(HierarchicalFiniteMachine, self).__init__(
@@ -375,11 +460,20 @@ class HierarchicalFiniteMachine(FiniteMachine):
 
     @classmethod
     def _effect_builder(cls, new_state, event):
-        return cls._Effect(new_state['reactions'].get(event),
-                           new_state["terminal"], new_state.get('machine'))
+        return cls.Effect(new_state['reactions'].get(event),
+                          new_state["terminal"], new_state.get('machine'))
 
     def add_state(self, state,
                   terminal=False, on_enter=None, on_exit=None, machine=None):
+        """Adds a given state to the state machine.
+
+        :param machine: the nested state machine that will be transitioned
+                        into when this state is entered
+        :type machine: :py:class:`.FiniteMachine`
+
+        Further arguments are interpreted as
+        for :py:meth:`.FiniteMachine.add_state`.
+        """
         if machine is not None and not isinstance(machine, FiniteMachine):
             raise ValueError(
                 "Nested state machines must themselves be state machines")
@@ -398,13 +492,45 @@ class HierarchicalFiniteMachine(FiniteMachine):
             c._nested_machines = self._nested_machines.copy()
         return c
 
-    def initialize(self, start_state=None):
+    def initialize(self, start_state=None,
+                   nested_start_state_fetcher=None):
+        """Sets up the state machine (sets current state to start state...).
+
+        :param start_state: explicit start state to use to initialize the
+                            state machine to. If ``None`` is provided then the
+                            machine's default start state will be used
+                            instead.
+        :param nested_start_state_fetcher: A callback that can return start
+                                           states for any nested machines
+                                           **only**. If not ``None`` then it
+                                           will be provided a single argument,
+                                           the machine to provide a starting
+                                           state for and it is expected to
+                                           return a starting state (or
+                                           ``None``) for each machine called
+                                           with. Do note that this callback
+                                           will also be passed to other nested
+                                           state machines as well, so it will
+                                           also be used to initialize any state
+                                           machines they contain (recursively).
+        """
         super(HierarchicalFiniteMachine, self).initialize(
             start_state=start_state)
         for data in six.itervalues(self._states):
             if 'machine' in data:
-                data['machine'].initialize()
+                nested_machine = data['machine']
+                nested_start_state = None
+                if nested_start_state_fetcher is not None:
+                    nested_start_state = nested_start_state_fetcher(
+                        nested_machine)
+                if isinstance(nested_machine, HierarchicalFiniteMachine):
+                    nested_machine.initialize(
+                        start_state=nested_start_state,
+                        nested_start_state_fetcher=nested_start_state_fetcher)
+                else:
+                    nested_machine.initialize(start_state=nested_start_state)
 
     @property
     def nested_machines(self):
+        """Dictionary of **all** nested state machines this machine may use."""
         return self._nested_machines
